#property strict

// Very crude benchmarking of insertion sort versus quicksort
// on an array of 10000 MqlRates (sorting by high price, desc).
// The sort functions are equivalent in that they are both
// templated, and the quicksort has been adapted to use
// the same sort-comparison object as the insertion-sort code.

// Note: quicksort does not necessarily preserve the original
// order for items which have the same value being compared.
// In other words, in this example: bars with equal high prices
// will be kept in the original open-time order by insertion-sort, 
// but may no longer be in time order after quicksort. To restore
// this, simply change the comparison function for quicksort
// so that it sorts by high and then by time. Still vastly
// faster than insertion-sort.

// Approximate count of number of milliseconds spent in each sort function
// (Values can go haywire every 24.9 days when the value of GetTickCount() wraps round!)
uint tcIStotal = 0, tcQStotal = 0;

// Un-comment the following line to write CSV files for comparison purposes
// #define WRITE_CSV


void OnStart()
{
   int UseBars = 10000;
   int NumberOfIterations = 1;
   
   for (int iter = 0; iter < NumberOfIterations; iter++) {
      Test_QuickSort(UseBars);
      Test_InsertionSort(UseBars);
   }
   
   Print("Insertion sort: " , tcIStotal, " total milliseconds");
   Print("Quicksort: " , tcQStotal, " total milliseconds");
}

// ******************************************************************************************
// Includes of external libaries for sorting
// ******************************************************************************************

#include <insertion_sort_1.mqh>

#include <quicksort.mqh>


// ******************************************************************************************
// Sorting condition used by insertion-sort
// ******************************************************************************************

class ByHighDescending{
public:
   bool is_before(const MqlRates& lhs, const MqlRates& rhs) {return lhs.high > rhs.high;}
};

// ******************************************************************************************
// Equivalent condition used by quicksort: identical apart from inheriting 
// from the abstract class
// ******************************************************************************************

class ByHighDescendingQS : public IQuicksortComparer<MqlRates> {
public:
   bool is_before(const MqlRates& lhs, const MqlRates& rhs) {return lhs.high > rhs.high;}
};


// ******************************************************************************************
// Identical test functions for insertion-sort and quicksort, getting the last 10000 
// bars (or however many available) and sorting them
// ******************************************************************************************

void Test_QuickSort(int UseBars)
{
   MqlRates arrBars[];
   int ct = CopyRates(Symbol(), Period(), 0, UseBars, arrBars);
   Print("Quick sort, bars available: " , ct);

   uint tcQSstart = GetTickCount();
   ByHighDescendingQS bhd;
   QuickSort_Start_Count(arrBars, 0, ct, bhd);
   uint tcQSend = GetTickCount();
   
   tcQStotal += (tcQSend - tcQSstart); 

   // Log sort results for comparion/debugging
   #ifdef WRITE_CSV
      int f = FileOpen("quicksort.csv", FILE_WRITE | FILE_TXT);
      for (int i = 0; i < ct; i++) {
         FileWriteString(f, TimeToString(arrBars[i].time) + "," + DoubleToString(arrBars[i].high, 5) + "\r\n");
      }
      FileClose(f);   
   #endif
}


void Test_InsertionSort(int UseBars)
{
   MqlRates arrBars[];
   int ct = CopyRates(Symbol(), Period(), 0, UseBars, arrBars);
   Print("Insertion sort, bars available: " , ct);

   uint tcISstart = GetTickCount();
   ByHighDescending bhd;
   insertion_sort(arrBars, 0, ct, bhd);
   uint tcISend = GetTickCount();
   
   tcIStotal += (tcISend - tcISstart); 
   
   // Log sort results for comparion/debugging
   #ifdef WRITE_CSV
      int f = FileOpen("insertionsort.csv", FILE_WRITE | FILE_TXT);
      for (int i = 0; i < ct; i++) {
         FileWriteString(f, TimeToString(arrBars[i].time) + "," + DoubleToString(arrBars[i].high, 5) + "\r\n");
      }
      FileClose(f);   
   #endif
}



